import unittest
from unittest.mock import patch
from flytekitplugins.k8sdataservice.k8s.kube_config import KubeConfig
from kubernetes.config import ConfigException


class TestKubeConfig(unittest.TestCase):

    @patch("flytekitplugins.k8sdataservice.k8s.kube_config.config.load_incluster_config")
    def test_load_kube_config_success(self, mock_load_incluster_config):
        kube_config = KubeConfig()
        kube_config.load_kube_config()
        mock_load_incluster_config.assert_called_once()

    @patch("flytekitplugins.k8sdataservice.k8s.kube_config.config.load_incluster_config")
    def test_load_kube_config_failure(self, mock_load_incluster_config):
        # Simulate a ConfigException
        mock_load_incluster_config.side_effect = ConfigException("In-cluster config not found.")
        kube_config = KubeConfig()

        with self.assertLogs('flytekit', level='WARNING') as log:
            kube_config.load_kube_config()
            self.assertEqual(f"WARNING:flytekit:Failed to load in-cluster configuration. In-cluster config not found.", log.output[-1])



from dataclasses import dataclass
from typing import Optional

from flyteidl.core.execution_pb2 import TaskExecution
from flytekitplugins.k8sdataservice.k8s.manager import K8sManager
from flytekitplugins.k8sdataservice.task import DataServiceConfig

from flytekit import logger
from flytekit.extend.backend.base_connector import AsyncConnectorBase, ConnectorRegistry, Resource, ResourceMeta
from flytekit.models.literals import LiteralMap
from flytekit.models.task import TaskTemplate


@dataclass
class DataServiceMetadata(ResourceMeta):
    dataservice_config: DataServiceConfig
    name: str


class DataServiceConnector(AsyncConnectorBase):
    name = "K8s DataService Async Connector"

    def __init__(self):
        self.k8s_manager = K8sManager()
        super().__init__(task_type_name="dataservicetask", metadata_type=DataServiceMetadata)
        self.config = None

    def create(
        self, task_template: TaskTemplate, output_prefix: str, inputs: Optional[LiteralMap] = None, **kwargs
    ) -> DataServiceMetadata:
        graph_engine_config = task_template.custom
        self.k8s_manager.set_configs(graph_engine_config)
        logger.info(f"Loaded connector config file {self.config}")
        existing_release_name = graph_engine_config.get("ExistingReleaseName", None)
        logger.info(f"The existing data service release name is {existing_release_name}")

        name = ""
        if existing_release_name is None or existing_release_name == "":
            logger.info("Creating K8s data service resources...")
            name = self.k8s_manager.create_data_service()
            logger.info(f'Data service {name} with image {graph_engine_config["Image"]} completed')
        else:
            name = existing_release_name
            logger.info(f"User configs to use the existing data service release name: {name}.")

        dataservice_config = DataServiceConfig(
            Name=graph_engine_config.get("Name", None),
            Image=graph_engine_config["Image"],
            Command=graph_engine_config["Command"],
            Cluster=graph_engine_config["Cluster"],
            ExistingReleaseName=graph_engine_config.get("ExistingReleaseName", None),
        )
        metadata = DataServiceMetadata(
            dataservice_config=dataservice_config,
            name=name,
        )
        logger.info(f"Created DataService metadata {metadata}")
        return metadata

    def get(self, resource_meta: DataServiceMetadata) -> Resource:
        logger.info("K8s Data Service get is called")
        data = resource_meta.dataservice_config
        data_dict = data.__dict__ if isinstance(data, DataServiceConfig) else data
        logger.info(f"The data_dict is {data_dict}")
        self.k8s_manager.set_configs(data_dict)
        name = data.Name
        logger.info(f"Get the stateful set name {name}")

        k8s_status = self.k8s_manager.check_stateful_set_status(name)
        flyte_state = None
        if k8s_status in ["failed", "timeout", "timedout", "canceled", "skipped", "internal_error"]:
            flyte_state = TaskExecution.FAILED
        elif k8s_status in ["done", "succeeded", "success"]:
            flyte_state = TaskExecution.SUCCEEDED
        elif k8s_status in ["running", "terminating", "pending"]:
            flyte_state = TaskExecution.RUNNING
        else:
            logger.error(f"Unrecognized state: {k8s_status}")
        outputs = {
            "data_service_name": name,
        }
        # TODO: Add logs for StatefulSet.
        return Resource(phase=flyte_state, outputs=outputs)

    def delete(self, resource_meta: DataServiceMetadata):
        logger.info("DataService delete is called")
        data = resource_meta.dataservice_config

        data_dict = data.__dict__ if isinstance(data, DataServiceConfig) else data
        self.k8s_manager.set_configs(data_dict)

        name = resource_meta.name
        logger.info(f"To delete the DataService (e.g., StatefulSet and Service) with name {name}")
        self.k8s_manager.delete_stateful_set(name)
        self.k8s_manager.delete_service(name)


ConnectorRegistry.register(DataServiceConnector())



from setuptools import find_namespace_packages, setup

PLUGIN_NAME = "k8sdataservice"

microlib_name = f"flytekitplugins-{PLUGIN_NAME}"

plugin_requires = ["flytekit>=1.11.0,<2.0.0", "kubernetes>=23.6.0,<24.0.0", "flyteidl>=1.11.0,<2.0.0"]

__version__ = "1.16.1b0"

setup(
    title="Kubernetes Stateful",
    title_expanded="K8s Stateful Service Plugin",
    name=microlib_name,
    version=__version__,
    author="LinkedIn",
    author_email="shuliang@linkedin.com",
    description="Flytekit K8s Data Service Plugin",
    namespace_packages=["flytekitplugins"],
    packages=find_namespace_packages(where="."),
    include_package_data=True,
    install_requires=plugin_requires,
    license="apache2",
    python_requires=">=3.9",
    classifiers=[
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    entry_points={"flytekit.plugins": [f"{PLUGIN_NAME}=flytekitplugins.{PLUGIN_NAME}"]},
)
